@@ -600,7 +600,7 @@ module NumRu
 	 self
       end
       def get_aux(name)
-	 @aux[name]
+	 @aux && @aux[name]
       end
       def aux_names
 	 @aux ? @aux.keys : []
@@ -890,7 +890,25 @@ module NumRu
       end
       private :_cut_
 
-      def _minloc_(a)  # private to be used in _cut_ 
+      # Find index nearest to loc
+      #
+      # * loc (Numeric, Date, DateTime)
+      #
+      # Return value: the index (Integer)
+      def nearest_index(loc)
+        if loc.class <= Date
+          loc = UNumeric.from_date(loc,pos.units,self.pos.get_att('calendar')).val
+        end
+        ax = self.pos.val
+        if cyclic_extendible?
+          dx, base = _cyclic_dx(ax-loc, modulo)
+        else
+          dx = (ax-loc).abs
+        end
+        idx = _minloc_(dx)
+      end
+
+      def _minloc_(a)  # private to be used in nearest_index (for cut etc)
         # Developper's MEMO: Such a method should be supported in NArray
         minloc = 0  # initialization
         for i in 0...(a.length-1)   # here, do not assume monotonic
@@ -1158,7 +1176,16 @@ if $0 == __FILE__
 
    p "## test join (cell)"
    axcel_c2 = Axis.new(true).set_cell(xc+10,xb+10)
+   axcel_c2.set_aux('aux1',xc*10)
    axcel_c2.set_pos_to_center
    jax = Axis.join([axcel_c,axcel_c2])
    p jax, jax.name, jax.pos.val.to_a, jax.cell_bounds.val.to_a
+
+   p "## test neaest index"
+   loc = 1.1
+   p axpt, loc, axpt.nearest_index(loc)
+   xc.put_att("modulo",[10.0])
+   axpt_cyc = Axis.new().set_pos(xc)
+   loc = -8.5
+   p axpt_cyc, axpt_cyc.modulo, loc, axpt.nearest_index(loc)
 end
