@@ -524,6 +524,26 @@
     RETURN VALUE
     * a GPhys (possibly self)
 
+---mask!(mask)
+    Mask the data by the mask array (additionally by the "&"
+    operation, if self has data missing). 
+    (Destructive version to replace the data array by the result.
+    -- replaced but not over-written.)
+
+    ARGUMENTS
+    * mask (byte type NArray or NArrayMiss)
+
+    RETURN VALUE
+    * self
+
+---mask(mask)
+    Same as mask! but self is preserved.
+
+    ARGUMENTS
+    * mask (byte type NArray or NArrayMiss)
+
+    RETURN VALUE
+    * a GPhys
 
 === Math functions (instance methods)
 
@@ -727,6 +747,9 @@ module NumRu
       def rank
 	@grid.rank
       end
+      def rank_len_ne_1
+	(shape_current - [1]).length
+      end
       def axis(i)
 	@grid.axis(i)
       end
@@ -785,7 +808,7 @@ module NumRu
       ## For graphics -->
       def first3D
 	raise "rank less than 3" if rank < 3
-	self[true,true,*([0]*(rank-3))]
+	self[true,true,true,*([0]*(rank-3))]
       end
       def first2D
 	raise "rank less than 2" if rank < 2
@@ -795,6 +818,31 @@ module NumRu
 	raise "rank less than 1" if rank < 1
 	self[true,*([0]*(rank-1))]
       end
+      def firstNDreally(nd)
+        sh = shape_current
+        n = 0
+        sel = sh.map do |len|
+          if len > 1 && n < nd
+            n += 1
+            true
+          else
+            0
+          end
+        end
+        if n<nd
+          raise "Invalid shape #{sh} (not really #{nd}D or greater): # of dims with len>1 is fewer than #{nd}"
+        end
+        self[*sel]
+      end
+      def first3Dreally
+        firstNDreally(3)
+      end
+      def first2Dreally
+        firstNDreally(2)
+      end
+      def first1Dreally
+        firstNDreally(1)
+      end
       ## <-- For graphics
 
       def coerce(other)
@@ -1102,6 +1150,29 @@ module NumRu
 
       end
 
+      def mask!(mask)
+        mask = mask.to_na(0) if mask.is_a?(NArrayMiss)
+        v = self.val
+        if v.is_a?(NArrayMiss)
+          v.set_mask( v.get_mask & mask )
+        else
+          v = NArrayMiss.to_nam(v,mask)
+        end        
+        self.replace_val(v)
+      end
+
+      def mask(mask)
+        mask = mask.to_na(0) if mask.is_a?(NArrayMiss)
+        v = self.val
+        if v.is_a?(NArrayMiss)
+          v.set_mask( v.get_mask & mask )
+        else
+          v = NArrayMiss.to_nam(v,mask)
+        end        
+        self.copy.replace_val(v)
+      end
+
+
       def marshal_dump
         [@data.copy, @grid.copy]
       end
@@ -1472,20 +1543,25 @@ module NumRu
 	 rtable.reverse!
 
 	 if table != rtable
-	    # < matching ambiguous => try to match by name >
+	    # < matching ambiguous => match by name where ambiguous >
 
-	    real_table = table.dup  # just to start with.
-                                    # rtable will be merged in the following
-
-	    shs.each_index do |i|
-	       #print axnms[i]," ",axnml[ table[i] ]," ",axnml[ rtable[i] ],"\n"
-	       if axnms[i] == axnml[ rtable[i] ]
-		  real_table[i] = rtable[i]
-	       elsif  axnms[i] != axnml[ table[i] ]
-		  raise "Both matchings by orders and by names failed for the #{i}-th axis #{axnms[i]}."
-	       end
-	    end
-	    
+	    real_table = Array.new
+            for i in 0...shs.length
+              if table[i] == rtable[i]
+                real_table[i] = table[i]
+              else
+                for j in table[i]..rtable[i]
+                  if axnms[i] == axnml[j] && shs[i] == shl[j]
+                    real_table[i] = j
+                    break
+                  end
+                end
+                if real_table[i].nil?
+                  raise("Matching failed: #{shl.inspect}(#{axnml.inspect})" +
+                        " and #{shs.inspect}(#{axnms.inspect})")
+                end
+              end
+            end
 	    table = real_table
 
 	 end
